use std::collections::HashSet;

use libp2p::gossipsub::TopicHash;

use crate::plugin::Identifier;
use crate::stingray_sdk::LuaRef;

#[derive(Debug)]
pub(crate) struct RPC {
    listeners: HashSet<LuaRef>,
}

impl RPC {
    pub fn new() -> Self {
        Self {
            listeners: Default::default(),
        }
    }

    pub(crate) fn add_listener(&mut self, fn_ref: LuaRef) {
        self.listeners.insert(fn_ref);
    }

    pub(crate) fn listeners(&self) -> impl Iterator<Item = &LuaRef> {
        self.listeners.iter()
    }
}

#[derive(Debug)]
pub(crate) struct RPCMessage {
    pub topic: TopicHash,
    pub data: Vec<u8>,
}

impl RPCMessage {
    pub(crate) fn new(id: Identifier, data: impl Into<Vec<u8>>) -> Self {
        let topic = id.to_topic();
        Self {
            topic,
            data: data.into(),
        }
    }
}
